<?php

namespace OPTN;

use WP_Error;
use WP_REST_Request;
use WP_REST_Response;

final class AdminRest {

	/**
	 * Database instance
	 *
	 * @var \OPTN\DB
	 */
	private $db;

	private $fluent_crm_fields = array(
		array(
			'value' => 'prefix',
			'label' => 'Name Prefix',
		),
		array(
			'value' => 'first_name',
			'label' => 'First Name',
		),
		array(
			'value' => 'last_name',
			'label' => 'Last Name',
		),
		array(
			'value' => 'full_name',
			'label' => 'Full Name',
		),
		array(
			'value' => 'email',
			'label' => 'Email',
		),
		array(
			'value' => 'timezone',
			'label' => 'Timezone',
		),
		array(
			'value' => 'address_line_1',
			'label' => 'Address Line 1',
		),
		array(
			'value' => 'address_line_2',
			'label' => 'Address Line 2',
		),
		array(
			'value' => 'city',
			'label' => 'City',
		),
		array(
			'value' => 'state',
			'label' => 'State',
		),
		array(
			'value' => 'postal_code',
			'label' => 'Postal Code',
		),
		array(
			'value' => 'country',
			'label' => 'Country',
		),
		// array(
		// 'value' => 'ip',
		// 'label' => 'IP Address',
		// ),
		array(
			'value' => 'phone',
			'label' => 'Phone',
		),
		array(
			'value' => 'source',
			'label' => 'Source',
		),
		array(
			'value' => 'date_of_birth',
			'label' => 'Date of Birth',
		),
	);


	public function __construct() {
		$this->db = \OPTN\DB::get_instance();
		$this->register_routes();
	}

	public function register_routes() {
		\OPTN\Analytics::get_instance()->register_admin_routes();

		register_rest_route(
			'optn/v1',
			'/get-post',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'handle_get_post' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);

		register_rest_route(
			'optn/v1',
			'/save-post',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'handle_save_post' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);

		register_rest_route(
			'optn/v1',
			'/delete-post',
			array(
				'methods'             => 'DELETE',
				'callback'            => array( $this, 'handle_delete_post' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);

		register_rest_route(
			'optn/v1',
			'/restore-post',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'handle_restore_post' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);

		register_rest_route(
			'optn/v1',
			'/get-posts',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'handle_get_posts' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);

		register_rest_route(
			'optn/v1',
			'/get-leads',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'handle_get_leads' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);

		register_rest_route(
			'optn/v1',
			'/delete-leads',
			array(
				'methods'             => 'DELETE',
				'callback'            => array( $this, 'handle_delete_leads' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);

		register_rest_route(
			'optn/v1',
			'/get-fluentcrm-info',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'handle_get_fluentcrm_info' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);

		register_rest_route(
			'optn/v1',
			'/get-products',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'handle_get_products' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);

		register_rest_route(
			'optn/v1',
			'/get-archives',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'handle_get_archive_pages' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);

		register_rest_route(
			'optn/v1',
			'/get-rule-options',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'handle_get_rule_options' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);

		register_rest_route(
			'optn/v1',
			'/get-rule-values',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'handle_get_rule_values' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);

		register_rest_route(
			'optn/v1',
			'/get-templates',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'handle_get_templates' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);
	}

	public function handle_get_post( WP_REST_Request $req ) {
		$id = isset( $req['id'] ) ? intval( $req['id'] ) : null;

		$res = $this->db->get_saved_post( $id );

		return rest_ensure_response( $res );
	}

	public function handle_save_post( WP_REST_Request $req ) {
		$req_type = isset( $req['req_type'] ) ? sanitize_text_field( $req['req_type'] ) : null;

		$title       = isset( $req['title'] ) ? sanitize_text_field( $req['title'] ) : null;
		$data   = isset( $req['data'] ) && \OPTN\Helper::is_valid_json( $req['data'] ) ? $req['data'] : null; // phpcs:ignore
		$type        = isset( $req['type'] ) ? sanitize_text_field( $req['type'] ) : null;
		$id          = isset( $req['id'] ) ? intval( $req['id'] ) : null;
		$template_id = isset( $req['templateId'] ) ? intval( $req['templateId'] ) : null;
		$status      = isset( $req['status'] ) ? ( sanitize_text_field( $req['status'] ) === 'publish' || true === $req['status'] || 1 === $req['status'] ? true : false ) : false;

		$res = null;
		if ( $req_type === 'create' ) {
			$res = $this->db->create_new_post( $type, $id, $title, $template_id );
		} elseif ( $req_type === 'get' ) {
			$res = $this->db->get_saved_post( $id );
		} elseif ( $req_type === 'update' ) {
			$res = $this->db->update_saved_post( $id, $title, $status, $data, $type );
		}

		if ( isset( $res['error'] ) ) {
			return rest_ensure_response( new WP_Error( 'optin_invalid_request', $res['error'], array( 'status' => 400 ) ) );
		}

		return rest_ensure_response( $res );
	}

	public function handle_get_rule_values( WP_REST_Request $req ) {
		$type    = isset( $req['type'] ) ? sanitize_text_field( $req['type'] ) : null;
		$search  = isset( $req['search'] ) ? sanitize_text_field( $req['search'] ) : '';
		$exclude = isset( $req['exclude'] ) && is_array( $req['exclude'] ) ? array_map( 'absint', $req['exclude'] ) : array();

		$res = \OPTN\DisplayRules::get_values( $type, $search, $exclude );

		return rest_ensure_response( $res );
	}

	public function handle_get_rule_options( WP_REST_Request $req ) {
		$type = isset( $req['type'] ) ? sanitize_text_field( $req['type'] ) : null;

		$res = \OPTN\DisplayRules::get_options( $type );

		return rest_ensure_response( $res );
	}

	public function handle_get_posts( WP_REST_Request $req ) {
		$status = isset( $req['status'] ) ? sanitize_text_field( $req['status'] ) : 'all';
		$limit  = isset( $req['limit'] ) ? intval( $req['limit'] ) : 12;
		$page   = isset( $req['page'] ) ? ( intval( $req['page'] ) > 0 ? intval( $req['page'] ) : 1 ) : 1;
		$search = isset( $req['search'] ) ? sanitize_text_field( $req['search'] ) : '';

		return rest_ensure_response( $this->db->get_posts( $status, $limit, $page, $search ) );
	}

	public function handle_delete_post( WP_REST_Request $req ) {

		// Validation/Sanitization
		$ids = isset( $req['ids'] ) ? $req['ids'] : null;
		if ( ! is_array( $ids ) ) {
			return rest_ensure_response( new WP_Error( 'optin_invalid_request', __( 'Post ID(s) is required', 'optin' ), array( 'status' => 400 ) ) );
		}
		$ids = array_map( 'absint', $ids );

		// Validation/Sanitization
		$type = isset( $req['type'] ) ? sanitize_text_field( $req['type'] ) : null;
		if ( $type !== 'soft' && $type !== 'hard' ) {
			return rest_ensure_response( new WP_Error( 'optin_invalid_request', __( 'Invalid Delete Type', 'optin' ), array( 'status' => 400 ) ) );
		}

		$res = $this->db->delete_post( $ids, $type );

		return rest_ensure_response( $res );
	}


	public function handle_restore_post( WP_REST_Request $req ) {
		// Validation/Sanitization.
		$ids = isset( $req['ids'] ) ? $req['ids'] : null;
		if ( ! is_array( $ids ) ) {
			return rest_ensure_response( new WP_Error( 'optin_invalid_request', __( 'Post ID(s) is required', 'optin' ), array( 'status' => 400 ) ) );
		}
		$ids = array_map( 'absint', $ids );

		$res = $this->db->restore_post( $ids );

		return rest_ensure_response( $res );
	}

	public function handle_get_leads( WP_REST_Request $req ) {
		// $status = isset( $req['status'] ) ? sanitize_text_field( $req['status'] ) : 'all';
		$limit = isset( $req['limit'] ) ? intval( $req['limit'] ) : 12;
		$page  = isset( $req['page'] ) ? ( intval( $req['page'] ) > 0 ? intval( $req['page'] ) : 1 ) : 1;
		// $search = isset( $req['search'] ) ? sanitize_text_field( $req['search'] ) : '';

		return rest_ensure_response( $this->db->get_leads( $limit, $page ) );
	}

	public function handle_delete_leads( WP_REST_Request $req ) {

		// Validation/Sanitization
		$ids = isset( $req['ids'] ) ? $req['ids'] : null;
		if ( ! is_array( $ids ) ) {
			return rest_ensure_response( new WP_Error( 'optin_invalid_request', __( 'Lead ID(s) is required', 'optin' ), array( 'status' => 400 ) ) );
		}
		$ids = array_map( 'absint', $ids );

		$res = $this->db->delete_leads( $ids );

		if ( isset( $res['error'] ) ) {
			return rest_ensure_response( new WP_REST_Response( 'error', $res['error'] ) );
		}

		return rest_ensure_response( new WP_REST_Response( 'success', 200 ) );
	}


	public function handle_get_products() {
		$active   = false;
		$products = array();

		if ( function_exists( 'wc_get_products' ) ) {
			$active = true;

			$products = wc_get_products(
				array(
					'limit'  => -1,
					'status' => 'publish',
				)
			);

			$products = array_map(
				function ( $product ) {
					return array(
						'value' => 'P' . $product->get_id(),
						'label' => '[WOO] ' . $product->get_name() . ' | ID: ' . $product->get_id(),
					);
				},
				$products
			);

		}

		$edd_products = get_posts(
			array(
				'post_type'      => 'download',
				'posts_per_page' => -1,
				'post_status'    => 'publish',
			)
		);

		$edd_products = array_map(
			function ( $product ) {
				return array(
					'value' => 'P' . $product->ID,
					'label' => '[EDD] ' . $product->post_title . ' | ID: ' . $product->ID,
				);
			},
			$edd_products
		);

		$products = array_merge( $products, $edd_products );

		return new \WP_REST_Response(
			array(
				'products' => $products,
				'active'   => $active,
			),
			200
		);
	}

	public function handle_get_archive_pages() {

		$archive_data = array();

		$post_type = get_post_types( array( 'public' => true ), 'objects' );

		foreach ( $post_type as $key => $type ) {

			if ( 'conversion_popup' === $type->name ) {
				continue;
			}

			// Taxonomy.
			$taxonomy = get_object_taxonomies( $type->name, 'objects' );
			if ( ! empty( $taxonomy ) ) {
				foreach ( $taxonomy as $tax ) {
					$args         = array(
						'taxonomy'   => $tax->name,
						'fields'     => 'all',
						'hide_empty' => false,
					);
					$post_results = get_terms( $args );
					if ( ! empty( $post_results ) ) {
						foreach ( $post_results as $key => $val ) {
							$archive_data[] = array(
								'value' => 'A' . $val->term_id,
								'label' => '[' . $type->label . '] ' . $val->name . ' | Term ID: ' . $val->term_id,
							);
						}
					}
				}
			}
		}

		return new \WP_REST_Response(
			array(
				'archive_pages' => $archive_data,
			),
			200
		);
	}

	public function handle_get_fluentcrm_info() {
		if ( ! function_exists( 'FluentCrmApi' ) || ! function_exists( 'fluentcrm_get_custom_contact_fields' ) ) {
			return new \WP_REST_Response(
				array(
					'active' => false,
				),
				200
			);
		}

		$lists  = FluentCrmApi( 'lists' )->get()->toArray();
		$tags   = FluentCrmApi( 'tags' )->get()->toArray();
		$fields = fluentcrm_get_custom_contact_fields();

		$lists = array_map(
			function ( $flist ) {
				return array(
					'value' => $flist['id'],
					'label' => $flist['title'],
				);
			},
			$lists
		);

		$tags = array_map(
			function ( $tag ) {
				return array(
					'value' => $tag['id'],
					'label' => $tag['title'],
				);
			},
			$tags
		);

		$fields = array_map(
			function ( $item ) {
				return array(
					'value' => $item['slug'],
					'label' => $item['label'],
				);
			},
			$fields
		);

		$fields = array_merge( $this->fluent_crm_fields, $fields );

		return new \WP_REST_Response(
			array(
				'lists'  => $lists,
				'tags'   => $tags,
				'fields' => $fields,
				'active' => true,
			),
			200
		);
	}

	public function handle_get_templates() {
		return rest_ensure_response( \OPTN\Templates::get_templates() );
	}

	public function permission_callback() {
		// return true;
		return current_user_can( OPTN_MIN_CAPABILITY );
	}
}
