<?php

namespace OPTN;

class Templates {

	const URL = 'http://apps.wowoptin.com/wp-json/otmp/v1/template';

	public static function get_templates() {
		return self::fetch_templates_data();
	}

	private static function fetch_templates_data() {
		$key = get_option( 'optn_temp_metadata_key', '' );

		$cached_data = self::get_from_cache();

		$key = ! empty( $cached_data ) ? $key : '';

		$response = wp_remote_post(
			self::URL,
			array(
				'body'    => wp_json_encode(
					array(
						'key'     => $key,
						'version' => OPTN_VERSION,
					)
				),
				'timeout' => 15,
				'headers' => array(
					'content-type' => 'application/json',
				),
			)
		);

		if ( is_wp_error( $response ) ) {
			$error_message = $response->get_error_message();
			Helper::log_to_file( $error_message );
			return null;
		}

		$data = json_decode( $response['body'] );

		if ( isset( $data->data ) && true === $data->data ) {
			return $cached_data;
		}

		if ( ! isset( $data->data ) || ! isset( $data->key ) ) {
			return null;
		}

		update_option( 'optn_temp_metadata_key', $data->key );
		self::store_in_cache( $data->data->data );

		return $data->data->data;
	}

	public static function fetch_single_template( $id ) {
		$response = wp_remote_post(
			self::URL,
			array(
				'body'    => wp_json_encode(
					array(
						'id' => $id,
					)
				),
				'timeout' => 15,
				'headers' => array(
					'content-type' => 'application/json',
				),
			)
		);

		if ( is_wp_error( $response ) ) {
			$error_message = $response->get_error_message();
			Helper::log_to_file( $error_message );
			return null;
		}

		$data = json_decode( $response['body'] );

		return $data;
	}

	private static function get_path() {
		$upload_dir = wp_upload_dir();
		return trailingslashit( $upload_dir['basedir'] ) . 'optn';
	}

	private static function get_from_cache() {
		$path = self::get_path() . '/templates.json';

		if ( file_exists( $path ) ) {
			return wp_json_file_decode( $path );
		}

		return null;
	}

	private static function delete_from_cache() {
		$path = self::get_path() . '/templates.json';
		wp_delete_file( $path );
	}

	private static function store_in_cache( $content ) {
		if ( ! function_exists( 'wp_filesystem' ) ) {
			require_once ABSPATH . 'wp-admin/includes/file.php';
		}

		global $wp_filesystem;
		WP_Filesystem();

		$dir = self::get_path();

		if ( ! $wp_filesystem->is_dir( $dir ) ) {
			$wp_filesystem->mkdir( $dir );
		}

		$path = trailingslashit( $dir ) . 'templates.json';

		if ( ! $wp_filesystem->put_contents( $path, wp_json_encode( $content ), FS_CHMOD_FILE ) ) {
			Helper::log( 'Failed to write to the cache file: ' . $path );
		}
	}
}
