<?php

namespace OPTN;

use DateTime;

final class Helper {

	private function __construct() {}

	/**
	 * Logs via Query Monitor
	 *
	 * @param mixed $output
	 * @return void
	 */
	public static function log( $output ) {
		if ( class_exists( '\QM' ) && wp_get_environment_type() === 'local' ) {
			\QM::debug( $output );
		}
	}


	/**
	 * Logs via debug.log
	 *
	 * @param mixed $output
	 * @return void
	 */
	public static function log_to_file( $output ) {
		if ( wp_get_environment_type() === 'local' &&
			defined( 'WP_DEBUG' ) &&
			defined( 'WP_DEBUG_LOG' ) &&
			WP_DEBUG === true &&
			WP_DEBUG_LOG === true
		) {
			error_log( print_r( $output, true ) ); // phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_error_log,WordPress.PHP.DevelopmentFunctions.error_log_print_r
		}
	}

	/**
	 * Checks if prefix exists
	 *
	 * @param string  $value
	 * @param string  $prefix
	 * @param boolean $remove Removes the prefix if found.
	 * @return boolean
	 */
	public static function check_prefix( &$value, $prefix, $remove = false ) {
		$i = strpos( $value, $prefix );

		if ( $i !== false ) {
			if ( $remove ) {
				$value = substr( $value, strlen( $prefix ), strlen( $value ) );
			}
			return true;
		}

		return false;
	}


	/**
	 * Converts a binary array to decimal. Array's last element will be LSB
	 *
	 * @param array Binary array
	 * @return int
	 */
	public static function bin_array_to_dec( $arr ) {
		$bin = '';
		foreach ( $arr as $a ) {
			$bin .= $a ? '1' : '0';
		}
		return bindec( $bin );
	}

	public static function get_diff_pct( $old, $new ) {
		$diff = abs( $old - $new );
		$avg  = ( $old + $new ) / 2;
		return $avg !== 0 ? round( ( $diff / $avg ) * 100 ) : 0;
	}

	public static function format_number( $num ) {
		$units = array( '', 'K', 'M', 'B', 'T' );
		for ( $i = 0; $num >= 1000; $i++ ) {
			$num /= 1000;
		}
		return round( $num, 1 ) . $units[ $i ];
	}

	public static function get_revenue( $order_type, $order_id ) {

		if ( $order_type == 0 ) {
			if ( function_exists( 'wc_get_order' ) ) {
				$order = wc_get_order( $order_id );
				return $order->get_subtotal() ?? 0;
			}
			return 0;
		} elseif ( $order_type == 1 ) {
			if ( class_exists( 'EDD_Payment' ) ) {
				$payment = new \EDD_Payment( $order_id );
				return $payment->subtotal ?? 0;
			}
			return 0;
		}

		return 0;
	}

	public static function get_past_dates( $days = 7 ) {
		$dates = array();
		$today = new DateTime();

		for ( $i = 0; $i < $days; $i++ ) {
			$date = clone $today;
			$date->modify( "-$i day" );
			$dates[] = $date->format( 'Y-m-d' );
		}

		return $dates;
	}

	public static function is_valid_json( $string ) {
		json_decode( $string );
		return json_last_error() === JSON_ERROR_NONE;
	}

	public static function is_pro_active() {
		$lic_data = get_option('edd_optin_license_data', array());
		return defined('OPTN_PRO_VERSION') && isset($lic_data['license']) && 'valid' === $lic_data['license'];
	}
}
