<?php

namespace OPTN;

use WP_REST_Request;
use WP_REST_Response;

final class RestPublic {

	/**
	 * Database instance
	 *
	 * @var \OPTN\DB
	 */
	private $db;

	public function __construct() {
		$this->db = \OPTN\DB::get_instance();
		$this->register_routes();
	}

	public function register_routes() {
		register_rest_route(
			'optn/v1',
			'/fluentcrm-hook',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'handle_fluent_hook' ),
				'permission_callback' => array( $this, 'permission_callback' ),
			)
		);
	}

	public function handle_fluent_hook( WP_REST_Request $req ) {

		$data = $req->get_json_params();

		$conv_id = isset( $data['conv_id'] ) ? intval( $data['conv_id'] ) : null;

		if ( empty( $conv_id ) ) {
			return rest_ensure_response( new WP_REST_Response( 'Invalid conv id', 400 ) );
		}

		unset( $data['conv_id'] );

		$data['ip'] = Utils::get_user_info()['ip'];

		if ( function_exists( 'FluentCrmApi' ) ) {

			$contactApi = FluentCrmApi( 'contacts' );
			$contact    = $contactApi->createOrUpdate( $data );

			if ( empty( $contact ) ) {
				return rest_ensure_response( new WP_REST_Response( 'Invalid data', 400 ) );
			}

			if ( $contact && $contact->status == 'pending' ) {
				$contact->sendDoubleOptinEmail();
			}

			// Add leads
			$leads_data = array();

			$leads_data['conv_id'] = $conv_id;

			$leads_data['email'] = isset( $data['email'] ) ? sanitize_email( $data['email'] ) : null;

			$name = null;

			if ( isset( $data['full_name'] ) ) {
				$name = sanitize_text_field( $data['full_name'] );
			} elseif ( isset( $data['name'] ) ) {
				$name = sanitize_text_field( $data['full_name'] );
			} elseif ( isset( $data['first_name'] ) || isset( $data['last_name'] ) ) {
				$first_name = ! empty( $data['first_name'] ) ? $data['first_name'] : '';
				$last_name  = ! empty( $data['last_name'] ) ? $data['last_name'] : '';
				$name       = sanitize_text_field( trim( $first_name . ' ' . $last_name ) );
				$name       = empty( $name ) ? null : $name;
			}

			$leads_data['name'] = $name;

			$leads_data['integration'] = 'fluentcrm';

			$leads_data['data'] = Utils::sanitize_json_array( $data );

			$this->db->add_lead( $leads_data );

			return rest_ensure_response( new WP_REST_Response( 'Success', 200 ) );
		}

		return rest_ensure_response( new WP_REST_Response( 'FluentCRM is not installed or active', 500 ) );
	}


	public function permission_callback() {
		return true;
	}
}
