<?php

namespace OPTN;

	/**
	 * Utils class for public
	 *
	 * @since      1.0.1
	 *
	 * @package    optin
	 * @subpackage optin/public
	 */
class Utils {


	public static function get_woo_info() {
		$products        = array();
		$total           = 0;
		$num_of_products = 0;
		$thank_you_page  = false;
		$cart_page       = function_exists( 'is_cart' ) && is_cart();
		$all_count       = 0;

		if ( is_plugin_active( 'woocommerce/woocommerce.php' ) ) {

			if ( function_exists( 'WC' ) && isset( WC()->cart ) ) {
				$products = array_values(
					array_map(
						function ( $cart_item ) {
							return strval( $cart_item['product_id'] );
						},
						WC()->cart->get_cart() ?? array()
					)
				);

				$num_of_products = WC()->cart->get_cart_contents_count();

				$total = WC()->cart->get_cart_contents_total();
			}

			if ( function_exists( 'is_order_received_page' ) ) {
				$thank_you_page = is_order_received_page();
			}

			if ( ! $thank_you_page ) {
				$thank_you_page = ! empty( get_the_ID() ) && get_the_ID() == get_option( 'woocommerce_thanks_page_id' );
			}

			if ( function_exists( 'wc_get_products' ) ) {
				$args = array(
					'status'       => 'publish',
					'stock_status' => 'instock',
					'return'       => 'ids',
					'limit'        => -1,
				);

				$all_count = count( wc_get_products( $args ) );
			}
		}

		return array(
			'num_of_products' => $num_of_products,
			'total'           => $total,
			'products'        => $products,
			'thank_you_page'  => $thank_you_page,
			'cart_page'       => $cart_page,
			'all_count'       => $all_count,
		);
	}

	public static function get_edd_info() {
		$products        = array();
		$total           = 0;
		$num_of_products = 0;
		$thank_you_page  = false;
		$cart_page       = false;
		$all_count       = 0;

		if ( is_plugin_active( 'easy-digital-downloads/easy-digital-downloads.php' ) ) {
			if ( function_exists( 'edd_get_cart_contents' ) ) {
				$products = array_map(
					function ( $cart_item ) {
						return strval( $cart_item['id'] );
					},
					edd_get_cart_contents()
				);
			}

			if ( function_exists( 'edd_get_cart_quantity' ) ) {
				$num_of_products = edd_get_cart_quantity();
			}

			if ( function_exists( 'edd_get_cart_total' ) ) {
				$total = edd_get_cart_total();
			}

			if ( function_exists( 'edd_is_success_page' ) ) {
				$thank_you_page = edd_is_success_page();
			}

			$edd_options = get_option( 'edd_settings' );
			$cart_page   = isset( $edd_options['purchase_page'] ) && get_the_ID() == $edd_options['purchase_page'];

			$args = array(
				'post_type'      => 'download',
				'post_status'    => 'publish',
				'fields'         => 'ids',
				'posts_per_page' => -1,
			);

			$query = new \WP_Query( $args );

			$all_count = $query->found_posts;
		}

		return array(
			'num_of_products' => $num_of_products,
			'total'           => $total,
			'products'        => $products,
			'thank_you_page'  => $thank_you_page,
			'cart_page'       => $cart_page,
			'all_count'       => $all_count,
		);
	}


	public static function get_audio_html( $id ) {
		$audio = get_post_meta( $id, 'optn_sound', true );
		if ( ! empty( $audio ) ) {
			return '<audio src="' . OPTN_URL . '/assets/sounds/' . $audio . '" type="audio/mp3"></audio>';
		} else {
			return '';
		}
	}

	public static function get_sanitized_query_parameters() {
		if ( isset( $_GET ) && is_array( $_GET ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended

			$ret = array();

			foreach ( $_GET as $key => $value ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				$ret[ $key ] = sanitize_text_field( $value );
			}

			return $ret;
		}

		return array();
	}


	public static function get_terms_by_id( $id ) {
		$terms = array();
		$pt    = get_post_type( $id );
		if ( $pt ) {
			$taxes = get_object_taxonomies( $pt, 'objects' );
			if ( ! empty( $taxes ) ) {
				foreach ( $taxes as $tax ) {
					$the_terms = get_the_terms( $id, $tax->name );
					if ( ! empty( $the_terms ) ) {
						foreach ( $the_terms as $term ) {
							$terms[] = $term->term_id;
						}
					}
				}
			}
		}
		return $terms;
	}


	public static function get_visitor_types() {
		$roles   = array();
		$roles[] = is_user_logged_in() ? 'logged_in' : 'logged_out';
		$roles[] = self::detect_new_or_returning_visitor();
		return $roles;
	}

	private static function detect_new_or_returning_visitor() {
		$cookie_name    = 'optn_visitor_type';
		$is_new_visitor = false;

		if ( ! isset( $_COOKIE[ $cookie_name ] ) ) {
			$is_new_visitor = true;
			setcookie( $cookie_name, 'returning', intval( time() + ( 100 * 365 * 24 * 60 * 60 ) ), '/' );
		}

		return $is_new_visitor ? 'new' : 'returning';
	}

	public static function check_math( $curr_value, $rule_value, $cond ) {
		switch ( $cond ) {
			case 'eq':
				return $curr_value == $rule_value;
			case 'gt':
				return $curr_value > $rule_value;
			case 'lt':
				return $curr_value < $rule_value;
			default:
				return false;
		}
	}

	public static function check_values( $curr_values, $rule_values, $cond, $all_count = 0 ) {
		if ( isset( $rule_values[0] ) && str_starts_with( $rule_values[0], 'all' ) ) {

			if ( is_array( $curr_values ) ) {
				switch ( $cond ) {
					case 'any':
						return true;
					case 'all':
						return $all_count === count( $curr_values );
					case 'not_all':
						return $all_count !== count( $curr_values );
					case 'not_any':
					default:
						return false;
				}
			}

			switch ( $cond ) {
				case 'any':
					return true;
				case 'not_any':
				case 'all':
				case 'not_all':
				default:
					return false;
			}
		}

		if ( is_array( $curr_values ) ) {
			$intersect = array_intersect( $curr_values, self::clean_values( $rule_values ) );

			switch ( $cond ) {
				case 'any':
					return ! empty( $intersect );
				case 'all':
					return count( $intersect ) === count( $rule_values );
				case 'not_all':
					return count( $intersect ) !== count( $rule_values );
				case 'not_any':
					return empty( $intersect );
			}
		}

		$inc = in_array( $curr_values, self::clean_values( $rule_values ) );

		if ( 'any' === $cond || 'all' === $cond ) {
			return $inc;
		}

		if ( 'not_any' === $cond || 'not_all' === $cond ) {
			return ! $inc;
		}

		return false;
	}

	public static function clean_values( $values ) {
		if ( is_array( $values ) ) {
			return array_map(
				function ( $value ) {
					if ( strpos( $value, '###' ) !== false ) {
						return explode( '###', $value )[0];
					}
					return strval( $value );
				},
				$values
			);
		}

		if ( ! is_string( $values ) ) {
			return strval( $values );
		}
	}

	public static function maybe_remove_prefix( &$string, $prefix ) {
		if ( strpos( $string, $prefix ) === 0 ) {
			$string = substr( $string, strlen( $prefix ) );
			return true;
		}
		return false;
	}


	public static function get_user_info() {

		$user_info = array(
			'ip'      => null,
			'country' => null,
		);

		if ( isset( $_COOKIE['optn_user_info'] ) ) {
			$info                 = json_decode( stripslashes( $_COOKIE['optn_user_info'] ) );
			$user_info['ip']      = isset( $info->ip ) ? sanitize_text_field( $info->ip ) : null;
			$user_info['country'] = isset( $info->country ) ? sanitize_text_field( $info->country ) : null;
		} else {
			$user_info['ip'] = self::get_ip_from_request();
		}

		return $user_info;
	}

	public static function get_ip_from_request() {
		foreach ( array( 'HTTP_CLIENT_IP', 'HTTP_X_FORWARDED_FOR', 'HTTP_X_FORWARDED', 'HTTP_X_CLUSTER_CLIENT_IP', 'HTTP_FORWARDED_FOR', 'HTTP_FORWARDED', 'REMOTE_ADDR' ) as $key ) {
			if ( array_key_exists( $key, $_SERVER ) === true ) {
				foreach ( array_map( 'trim', explode( ',', $_SERVER[ $key ] ) ) as $ip ) {
					if ( filter_var( $ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE ) !== false ) {
						return $ip;
					}
				}
			}
		}

		return null;
	}

	public static function get_embed_hook_and_pos( $opt ) {
		$res = array( 'pos' => null );

		if ( self::maybe_remove_prefix( $opt, 'before_' ) ) {
			$res['pos'] = 'before';
		} elseif ( self::maybe_remove_prefix( $opt, 'after_' ) ) {
			$res['pos'] = 'after';
		} elseif ( self::maybe_remove_prefix( $opt, 'custom###' ) ) {

			if ( self::maybe_remove_prefix( $opt, 'before###' ) ) {
				$res['pos']  = 'before';
				$res['hook'] = $opt;
			} elseif ( self::maybe_remove_prefix( $opt, 'after###' ) ) {
				$res['pos']  = 'after';
				$res['hook'] = $opt;
			}

			return $res;
		}

		switch ( $opt ) {
			case 'body':
				$res['hook']   = 'wp_body_open';
				$res['single'] = false;
				break;
			case 'title':
				$res['hook']   = 'the_title';
				$res['single'] = true;
				break;
			case 'content':
				$res['hook']   = 'the_content';
				$res['single'] = true;
				break;
			case 'comment_form':
				if ( 'before' === $res['pos'] ) {
					$res['hook']   = 'comment_form_before';
					$res['single'] = true;
				} elseif ( 'after' === $res['pos'] ) {
					$res['hook']   = 'comment_form_after';
					$res['single'] = true;
				}
				break;
			case 'footer':
				$res['hook']   = 'the_footer';
				$res['single'] = false;
				break;
			default:
				$res['hook'] = null;
		}

		return $res;
	}

	/**
	 * Sanitizes a JSON array for safe use in WordPress
	 *
	 * @param string|array $input Raw JSON string or PHP array to sanitize
	 * @param array        $allowed_types Array of allowed data types ('string', 'int', 'float', 'bool', 'array')
	 * @param bool         $strict_checking Whether to error on unexpected fields/types
	 * @return array|WP_Error Sanitized array or WP_Error on failure
	 */
	public static function sanitize_json_array( $input, $allowed_types = array( 'string', 'int', 'float', 'bool', 'array', 'NULL' ), $strict_checking = false ) {
		// If input is JSON string, decode it first
		if ( is_string( $input ) ) {
			$decoded = json_decode( $input, true );
			if ( json_last_error() !== JSON_ERROR_NONE ) {
				return new \WP_Error(
					'invalid_json',
					'Invalid JSON format: ' . json_last_error_msg()
				);
			}
			$input = $decoded;
		}

		// Verify we have an array
		if ( ! is_array( $input ) ) {
			return new \WP_Error(
				'invalid_input',
				'Input must be a JSON string or array'
			);
		}

		// Recursive function to sanitize array values
		$sanitize_value = function ( $value, $key = '' ) use ( &$sanitize_value, $allowed_types, $strict_checking ) {
			$type = gettype( $value );

			// Handle nested arrays recursively
			if ( is_array( $value ) ) {
				if ( ! in_array( 'array', $allowed_types ) ) {
					return new \WP_Error(
						'invalid_type',
						sprintf( 'Array type not allowed for key "%s"', $key )
					);
				}
				$sanitized = array();
				foreach ( $value as $k => $v ) {
					// Sanitize array keys
					$k = sanitize_key( $k );

					$result = $sanitize_value( $v, $k );
					if ( is_wp_error( $result ) ) {
						return $result;
					}

					if ( null !== $result ) {
						$sanitized[ $k ] = $result;
					}
				}
				return $sanitized;
			}

			// Handle scalar values
			switch ( $type ) {
				case 'string':
					if ( ! in_array( 'string', $allowed_types ) ) {
						return new \WP_Error(
							'invalid_type',
							sprintf( 'String type not allowed for key "%s"', $key )
						);
					}
					return sanitize_text_field( $value );

				case 'integer':
					if ( ! in_array( 'int', $allowed_types ) ) {
						return new \WP_Error(
							'invalid_type',
							sprintf( 'Integer type not allowed for key "%s"', $key )
						);
					}
					return intval( $value );

				case 'double':
					if ( ! in_array( 'float', $allowed_types ) ) {
						return new \WP_Error(
							'invalid_type',
							sprintf( 'Float type not allowed for key "%s"', $key )
						);
					}
					return floatval( $value );

				case 'boolean':
					if ( ! in_array( 'bool', $allowed_types ) ) {
						return new \WP_Error(
							'invalid_type',
							sprintf( 'Boolean type not allowed for key "%s"', $key )
						);
					}
					return (bool) $value;
				case 'NULL':
					if ( ! in_array( 'NULL', $allowed_types ) ) {
						return new \WP_Error(
							'invalid_type',
							sprintf( 'Null type not allowed for key "%s"', $key )
						);
					}
					return null;

				default:
					if ( $strict_checking ) {
						return new \WP_Error(
							'invalid_type',
							sprintf( 'Unsupported type "%s" for key "%s"', $type, $key )
						);
					}
					// In non-strict mode, convert to sanitized string
					return sanitize_text_field( (string) $value );
			}
		};

		// Perform the sanitization
		$result = $sanitize_value( $input );

		// Check if we got an error
		if ( is_wp_error( $result ) ) {
			Helper::log_to_file( $result );
			return null;
		}

		return $result;
	}
}
